function getUniqueValues(count, min, max) {
    const values = [];
    while (values.length < count) {
        const randomValue = Math.floor(Math.random() * (max - values.length - min)) + min;
        values.push(values.slice().sort((a, b) => a - b).reduce((newValue, oldValue) => ((newValue >= oldValue) ? (newValue + 1) : newValue), randomValue));
    }
    return values;
}

function generateQuestion(winkel) {
    const useSin = (Math.random() < 0.5);
    const frage = {
        "html": "Für welche andere Winkel aus <code>[0°; 360°[</code> hat " + (useSin ? 'Sinus' : 'Kosinus') + " denselben Betrag wie <code>" + winkel + "°</code>?",
        "keepOrder": true,
        "singlechoice": true,
        "choices": [
            { "html": "Anderer Winkel: ", "unit": "°", "type": "number", size: 3, min: 0, max: 359, step: 1 },
            { "html": "Es gibt keinen" }
        ]
    }
    frage.response = 'Prinzipiell in Frage kommen (wie immer) die Winkel ';
    frage.response += '<code>180° - ' + winkel + '° = ' + (180 - winkel) + '°</code>';
    if (winkel > 180)
        frage.response += ' (entspricht <code>360° - ' + (winkel - 180) + '° = ' + (540 - winkel) + '°</code>)';
    frage.response += ', <code>180° + ' + winkel + '° = ' + (180 + winkel) + '°</code>';
    if (winkel > 180)
        frage.response += ' (entspricht <code>' + (winkel + 180) + '° - 360° = ' + (winkel - 180) + '°</code>)';
    frage.response += ' und <code>360° - ' + winkel + '° = ' + (360 - winkel) + '°</code>.';
    const andere = [360 - winkel, (540 - winkel) % 360, (540 + winkel) % 360]
        .filter((anderer, index, arr) => (anderer != winkel && arr.indexOf(anderer) == index))
        .filter(anderer => useSin ? ((anderer < 180) == (winkel < 180)) : ((Math.abs(180 - anderer) < 90) == (Math.abs(180 - winkel) < 90)));
    if (andere.length) {
        frage.choices[0].correct = true;
        frage.choices[0].accept = andere[0];
        frage.response += ' Davon passt aber nur ' + andere[0] + '°, weil nur davon der zugehörige Punkt auf dem Einheitskreis wie bei ' + winkel + '° ';
    } else {
        frage.choices[1].correct = true;
        frage.response += ' Aber bei diesem Winkel liegt der zugehörige Punkt auf dem Einheitskreis nicht wie bei ' + winkel + '° ';
    }
    if (useSin && (winkel < 180))
        frage.response += 'über der x-Achse';
    if (useSin && (winkel > 180))
        frage.response += 'unter der x-Achse';
    if (useSin && (winkel == 180))
        frage.response += 'auf der x-Achse';
    if (!useSin && (Math.abs(180 - winkel) < 90))
        frage.response += 'links der y-Achse';
    if (!useSin && (Math.abs(180 - winkel) > 90))
        frage.response += 'rechts der y-Achse';
    if (!useSin && (Math.abs(180 - winkel) == 90))
        frage.response += 'auf der y-Achse';
    frage.response += ' liegt.';
    return frage;
}

window.MultipleChoiceTestData = {
    // Optional: Wie viele Versuche hat der/die Lernende (Standardwert: Unendlich)
    "maxTries": Infinity,

    // Optional: Soll statt der Anzahl an nötigen Versuchen bewertet werden, wie gut der/die Lernende abschneidet?
    "reportScore": false,

    // Optional: Sollen die Fragen nicht durchgemischt werden?
    "keepOrder": false,

    // Optional: Wie soll bewertet werden?
    // "questions"  → Standardwert, Frage kann entweder richtig sein (1 von 1 Punkt) oder falsch (0 von 1 Punkt)
    // "average"    → Durchschnittlicher Prozentsatz an richtig ausgewählten bzw. richtig nicht ausgewählten Antwortmöglichkeiten pro auswählbaren Antwortmöglichkeiten bei jeder Frage
    // "choices"    → Anteil der richtig ausgewählten Antwortmöglichkeiten pro auswählbaren Antwortmöglichkeiten
    "countMode": "questions",

    // Wie viele Fragen sollen für einen Test gezogen werden?
    "questionsPerTest": 5,

    // Welcher Anteil muss mindestens richtig sein, um zu bestehen?
    "threshold": 1,

    // Optional: Begrüßungstext
    "htmlIntro": "<p>Trage den gesuchten Winkel aus <code>[0°; 360°[</code> ein:</p>",

    // Verfügbare Fragen
    "questions": [
    ],

    // Fragengeneratoren
    "generators": [
        arrFragen => getUniqueValues(5, 0, 71).forEach(winkel => arrFragen.push(generateQuestion(winkel * 5)))
    ]
}
